/*  web3js */
async function getWeb3() {
  return new Promise((resolve, reject) => {
    // Wait for loading completion to avoid race conditions with web3 injection timing.
    window.addEventListener("load", async () => {
      // Modern dapp browsers...
      if (window.ethereum) {
        const web3 = new Web3(window.ethereum);
        try {
          // Request account access if needed
          await window.ethereum.enable();
          // Acccounts now exposed
          resolve(web3);
        } catch (error) {
          reject(error);
        }
      }
      // Legacy dapp browsers...
      else if (window.web3) {
        // Use Mist/MetaMask's provider.
        const web3 = window.web3
        console.log("Injected web3 detected.");
        resolve(web3);
      }
      // Fallback to localhost; use dev console port by default...
      else {
        const provider = new Web3.providers.HttpProvider("http://127.0.0.1:8545");
        const web3 = new Web3(provider);
        console.log("No web3 instance injected, using Local web3.");
        resolve(web3);
      }
    });
  })
};

let contract, accounts;

/* contract */
async function getContract(contractData) {
  try {
    const web3 = await getWeb3();
    const networkId = await web3.eth.net.getId();
    accounts = await web3.eth.getAccounts();
    const deployedNetwork = contractData?.networks[networkId];
    contract = new web3.eth.Contract(contractData?.abi, deployedNetwork && deployedNetwork?.address);
  } catch (error) {
    console.error(error);
  }
}