#!/usr/bin/env node

const fs = require("fs");
const path = require("path");
const open = require("open");
const async = require("async");
const loadsh = require("loadsh");

const projectPath = __dirname?.slice(0, __dirname?.indexOf("node_modules") || 1);
let config = {};

const geConfig = () => {
	const files = fs.readdirSync(path.join(projectPath))
	if (files?.includes("converted.config.js")) {
		config = require(path.join(projectPath, "converted.config.js"))
	}
}
geConfig()

const { entry, out } = config;
const outPath = out || "./out"

const readFile = (file) => {
	try {
		const data = fs.readFileSync(path.join(projectPath, entry, file), "utf-8");
		return data;
	} catch (error) {
		return "";
	}
};

const writeFile = (file, data) => {
	fs.writeFileSync(path.join(projectPath, outPath, file), data);
};

const convertedJs = (files) => {
	let convertedAmount = 0, fileAmount = files?.length;
	return new Promise((resolve) => {
		async.forever(
			function (callback) {
				const fileName = loadsh?.camelCase(files[convertedAmount]?.split(".")[0])
				if (convertedAmount >= fileAmount) {
					console.log("转换JS成功");
					callback("done");
					resolve();
					return;
				}

				const content = readFile(`/${files[convertedAmount]}`)

				try {
					writeFile(`/js/${fileName}.js`, `const ${fileName} = ${content}`)
					convertedAmount++;
					callback();
				} catch (err) {
					console.log(err);
				}
			},
			function (err) {
				console.log(err);
			}
		);
	});
}

const getOtherHtmlStr = (content, name, fileName, callback) => {
	let htmlStr = `<!DOCTYPE html>
<html lang="en">
<meta http-equiv="content-type" content="text/html;charset=UTF-8" />   

<head>
    <meta charset="UTF-8" />
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Truffle Contract Page</title>
    <script src="../web3.js"></script>
    <script src="../js/${fileName}.js"></script>
    <script src="../contract.js"></script>
</head>

<body>
    <script>
        getContract(${fileName})
    </script>`


	let allFunArrList = `
	<script>
		let allFunArrList = [`

	const data = JSON.parse(content);
	const abi = data?.abi

	for (let i = 0; i < abi?.length; i++) {
		const item = abi[i];
		let itemHtml = `
    <div>
	`
		// 如果是事件和构造函数，那么就跳过
		if (item?.type === "event" || !item?.name) {
			continue
		}

		const inputs = item?.inputs
		// 需要参数的
		if (inputs?.length > 0) {
			itemHtml += `
        <script>
            async function get${item?.name}value() {
              `
			for (let j = 0; j < inputs?.length; j++) {
				itemHtml += `
              const dom${j} = document.getElementById("${item?.name}${inputs[j]?.name}paramas")
              if(dom${j}.value === ""){
                window.alert("${item?.name}值为空")
                return
              }
              `
			}

			itemHtml += `
              const data = await contract.methods.${item?.name}(`

			let inputStr = ""
			//拼接参数
			for (let j = 0; j < inputs?.length; j++) {
				itemHtml += `dom${j}.value,`
				inputStr += `
            <input id = "${item?.name}${inputs[j]?.name}paramas" placeholder="${inputs[j]?.name || inputs[j]?.type}" type="${inputs[j]?.type?.indexOf("uint") > -1 || inputs[j]?.type?.indexOf("int") > -1 ? "number" : "string"}"/>
            `
			}
			itemHtml += `)${item?.constant ? ".call()" : ".send({ from: accounts[0] })"};
        const dom = document.getElementById("${item?.name}");
		dom.innerHTML = ${!item?.constant ? true : "typeof data !== 'object' ? data : JSON.stringify(data)"} 
		getAllFunData()
        console.log("${item?.name}", data)
      }
        </script>
  <button style="margin: 10px" onclick="get${item?.name}value()">
    ${item?.name}
  </button>
`
			itemHtml += inputStr
			itemHtml += `
        <span id="${item?.name}">
                  </span>`
		} else {
			itemHtml += `
  <script>
  async function get${item?.name}value() {
  const data = await contract.methods.${item?.name}()${item?.constant ? ".call()" : ".send({ from: accounts[0] })"};
  const dom = document.getElementById("${item?.name}");
  dom.innerHTML = typeof data !== 'object' ? data : JSON.stringify(data);
  console.log("${item?.name}", data)
}
        </script>
  <button style="margin: 10px" onclick="get${item?.name}value()">
    ${item?.name}
  </button>
  <span id="${item?.name}">
  </span>
		`
			allFunArrList += !item?.constant ? "" : `get${item?.name}value,`
		}
		itemHtml += `
    </div >
  `

		htmlStr += itemHtml
	}

	allFunArrList += `]
	function getAllFunData(){
		for(let i=0; i<allFunArrList.length; i++){
			allFunArrList[i]()
		}
	}
	</script>
`
	htmlStr += allFunArrList

	htmlStr += `
</body >
  <html>`;

	writeFile(`/html/${fileName}.html`, htmlStr)
	callback();
}

const convertedHtml = (files) => {
	let convertedAmount = 0, fileAmount = files?.length;

	let indexHtml = `<!DOCTYPE html>
<html lang="en">
<meta http-equiv="content-type" content="text/html;charset=UTF-8" />

<head>
    <meta charset="UTF-8" />
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Truffle Contract  Page</title>
</head>

<body>
    <div>
    `
	async.forever(
		function (callback) {
			const name = files[convertedAmount]?.split(".")[0]
			const fileName = loadsh?.camelCase(name)

			if (convertedAmount >= fileAmount) {
				//open(path.join(projectPath, out, "./index.html"));
				console.log("转换HTML成功");
				indexHtml += `
    </div>
</body>

</html>`

				const contractData = fs.readFileSync(path.join(__dirname, "../public/contract.js"), "utf-8");
				const web3Data = fs.readFileSync(path.join(__dirname, "../public/web3.js"), "utf-8");
				writeFile(`/web3.js`, web3Data)
				writeFile(`/contract.js`, contractData)
				writeFile(`/index.html`, indexHtml)
				callback("done");
				return;
			}

			const content = readFile(`/${files[convertedAmount]}`)

			convertedAmount++;

			indexHtml += `
		<div><button><a href="./html/${fileName}.html">${fileName}</a></button></div>`

			getOtherHtmlStr(content, name, fileName, callback)
		},
		function (err) {
			console.log(err);
		}
	);
}

const converted = () => {
	const files = fs.readdirSync(path.join(projectPath, entry));
	const jsonFile = files?.filter(item => item?.includes("json"))

	try {
		fs.mkdirSync(path.join(projectPath, out))
		fs.mkdirSync(path.join(projectPath, out, "./js"))
		fs.mkdirSync(path.join(projectPath, out, "./html"))
	} catch {
		try {
			fs.mkdirSync(path.join(projectPath, out, "./js"))
			fs.mkdirSync(path.join(projectPath, out, "./html"))
		} catch {

		}
	}
	convertedJs(jsonFile)
	convertedHtml(jsonFile)
}

const start = () => {
	if (entry) {
		converted()
	} else {
		console.log("无合约JSON文件入口或html文件导出路径，请在truffle配置中补充完整")
	}
}

start()